/***********************************************************************
This file can be used as a startup template or as an include file.
The subroutines referenced are provided later in this file.

To initialize a real robot, your program should then do the following:
   gosub InitCommands
   rCommPort PORTNUM   // PORTNUM should be the # for your Bluetooth port
   rLocate 0,0
   gosub InitArloRobot  // or other initialization routine, 2 examples below
Note: The init routine may have to be modified to meet YOUR robot's needs
      For example, it may or may not have a compass, or different sensors
      
To make things EASY for those users that have an ARLO or an RB-9 robot,
you can simply do the following:
   rCommPort PORTNUM 
   rLocate ARLO,0      // you can also use RB9 in this way
This causes the RROS to setup all the parameters for either Arlo or the RB9
ALL EXCEPT the compass - because if it setup a compass, or the wrong compass
then the RROS will hang up.  for that reason, you should use the following 
command to THEN setup the compass of your choice, if you have one.
   rCommand(SetCompassType, TYPE) // TYPE should be HMC6352 or HMC5883
   
This new way of initializing is much faster than multiple rCommands.
Note: Some examples in our material utilize a separate file called
InitializationRoutines.  This example simply combines appropriate INIT 
routines into this file - feel free to do it either way, depending on
your needs and preferences.
************************************************************************/



// typically this file is included at the beginning of your program
// a call to InitRROScommands will set up constants that make working
// with rCommands easier and more readable
InitCommands:
InitRROScommands: 
  CalibServoDrive =    110
  CalibrateCompass =   111
  SetSenseInvMask =    112
  SetTurretOffset =    113
  SetRotationTime =    114   // calibrate for 90 degrees
  SetMoveTime =        115   // calibrate for radius distance
  SetLeftStopOffset =  116   // used to correct motor mismatch and centering
  SetRightStopOffset = 117
  SetTurnStyle =       118
  SetSpeed =           119
  SetCCdivisor  =      120   // used for wheel count correction
  SetMotorRamp =       121
  SetBumpDist =        122
  SetProxDist =        123
  SetExpanTimeout =    124
  SetRobotAngle =      125   // compensate for compass alignment
  SetBeaconTime =      126
  SetClicksPerDiam =   127
  SetClicksPer90 =     128
  // 129 used internally for rPen
  TurnToHalfAngle =    130
  PlaySound =          131   
  TestCommand =        132  // returns: argument, 1,2,3,4
  StartBalance =       133
  EnableMotors =       134
  EnableCounters =     135   // 1/0, counters always used for drift compensation 
  SetSlowDownSpeed =   136
  MuteBeaconDet =      137 
  MotorSetup =         138  
  SensorSetup =        139
  SetReducForwRight =  140
  SetReducBackRight =  141
  SetReducForwLeft =   142
  SetReducBackLeft =   143
  SetRROStimeout =     144
  SetCountDivisor =    145
  SetDriveServoWidth = 146
  FindBeacon =         147   // argument specifies direction of rotation
  SetBeaconDelay =     148   // delay till MIDDLE of first timing period
  SetSteerServoWidth = 149
  SetDriveServoDir =   150
  SetSteerServoDir =   151
  SetRobotDiameter =   152
  SetTurretServoWidth= 153
  SetTurnToMax =       154
  SetTurnToMin =       155
  SetSlowDown2 =       156
  SetPCBcalibrate =    157
  RROSversion =        158
  Drive =              159
  ReadRangers =        160
  SetCompassType =     161
  ForwardWithSenses =  162
  SetBeaconSpeed =     163
  // 160-199reserved space for main chip expansion codes

  // used only when an expansion chip is available
  ExpansionSetup =     200  // Sends initializing byte
  BecomeArmExpansion = 201  // typcially used internally only
  ReadExtSensors =     202  // reads bump, IR, line sensors from external source
                            // typically used only internally by RROS
  ReadArmSensors =     203  // reads available line sensors from Arm expansion
                            // typically use only internally
  // codes 204-209 are reserved for RROS use
  // all codes 210 - 255 are just passed to remote with all five bytes returned
  // codes 210-229 are reserved for user customized expansions
  SetNAVbeacons =      230
  ReadNAVangles =      231 
  ReadArmAnalogs =     232  // 2MSB specify #, all 8 bits passed on
  SetServoIndex =      233
  SetServoPosition =   234  // sets current position # to argument, increments #
  SetServoMin =        235  // sets current offset # to argument, increments #
  SetServoSpeed =      236  // sets current speed # to argument, increments #
  SetServoMax =        237  // sets max pulse width # (default 72), increments #
  EnableServos =       238  // 1 enables, 0 disables, default is disabled 
  GPSsetup =           239
  SetGPShighX =        240  
  SetGPSlowX =         241  // clears high byte automatically
  SetGPShighY =        242
  SetGPSlowY =         243  // clears high byte automatically 
  ExtSensorSetup =     244
  LookSetup =          245
  PenSetup =           246
  SetArmOutputs =      247
  SetNAVservoMinHigh = 248
  SetNAVservoMinLow =  249
  SetNAVservoMaxHigh = 250
  SetNAVservoMaxLow =  251

//  251-255 reserved for RROS use 

  
  // Sounds
  Pause =                0
  LowC =                 1
  LowD =                 2
  LowE =                 3
  LowF =                 4
  LowG =                 5
  LowA =                 6
  LowB =                 7
  MidC =                 8
  MidD =                 9
  MidE =                10
  MidF =                11
  MidG =                12
  MidA =                13
  MidB =                14
  HighC =               15
  HighD =               16
  HighE =               17
  HighF =               18
  HighG =               19
  HighA =               20
  HighB =               21
  Blip1 =               22
  Blip2 =               23
  InitTone =            24
  LowTone =             25
  BeepBeep =            26
  BeepBeepBeep =        27
  Phasor =              28
  Siren1 =              29
  Siren2 =              30
  Siren3 =              31
  Double =            7<<5
  Whole =             4<<5
  Half =              2<<5
  Quarter =           1<<5
  
  NORTH =                0  // to help with compass operations
  SOUTH =              180
  EAST =                90
  WEST =               270
  ARLO =               253  // These three optionally used as 1st param in rLocate
  RB9 =                254
  SlowComm =           255
  ForwRight =         0<<6
  ForwLeft =          1<<6
  RevRight =          2<<6
  RevLeft =           3<<6

  // Partial song for illustration purposes  
  Data Birthday; MidC|Quarter,MidC|Quarter,MidD|Half,MidC+Whole,MidF+Half,MidE+Half,Pause+Half
  Data Birthday; MidC+Quarter,MidC+Quarter,MidD+Half,MidC+Whole,MidG+Half,MidF+Half,Pause+Half
  Data Birthday; MidC+Quarter,MidC+Quarter,HighC+Half,MidA+Half,MidF+Half,MidE+Half,MidD+Whole,Pause+Half
  Data Birthday; MidB+Quarter,MidB+Quarter,MidA+Half,MidF+Half,MidG+Half,MidF+Whole
  Data Birthday; 0
  
  SMALLDC = 0
  SERVOMOTORS = 1
  ROBOCLAW = 2
  Steerable = 3
  IRSHORT = 0
  IRLONG = 1
  MAXBOTICS = 2
  PING = 3
  SR04 = 4
  SONIC = 4
  HMC6352 = 1<<3
  HMC5883 = 2<<3
  ENCODERS = 8     
  DIGITAL = 0
  FIVERANGE = 1<<6
  SIXRANGE = 2<<6
  GPS = 1
  LOOK = 2
  PEN = 4
  EXTSENSORS = 8
  ARM = 1<<4
  ARMwOUT = 2<<4
  ARMwNAV = 3<<4
return

PlayCurrentSong:
  NotePosition=0  
  while(1)
    if CurrentSong[NotePosition]=0 then break;
    x=rCommand(PlaySound,CurrentSong[NotePosition])
    NotePosition++
  wend
return 

InitRB9robot:
  SetTimeOut 50000
  rCommand(MotorSetup,SMALLDC+ENCODERS) 
  rCommand (SensorSetup,SR04+FIVERANGE)
  rCommand(SetMotorRamp,2)
  rCommand(SetClicksPerDiam,150)
  rCommand(SetClicksPer90,105) 
  rCommand(SetReducForwRight,0)
  rCommand(SetReducForwLeft,0)
  rCommand(SetReducBackRight,0)
  rCommand(SetReducBackLeft,0)
  rCommand(SetRotationTime,29) 
  rCommand(SetMoveTime,19)  
  rCommand(SetSpeed,25)
  rCommand(SetSlowDownSpeed,20) 
  rCommand(SetSlowDown2,15) 
  rCommand(SetCCdivisor,6)
  rCommand(SetBumpDist,10)
  rCommand(SetProxDist,20)
  rCommand(EnableCounters,1)
  rCommand(SetRobotAngle,90/2)
  rCommand(SetTurnStyle,50)
  rCommand(SetTurnToMax,20) // test
  rCommand(SetTurnToMin,15) // test
  rCommand(SetRROStimeout,15)
  rCommand(SetBeaconTime,35)
  rCommand(SetBeaconSpeed,30)
return

InitArloRobot:   // sample setup for our servomotor powered prototype
  SetTimeOut 30000
  rCommand(MotorSetup,SERVOMOTORS)
  rCommand(SensorSetup,SIXRANGE+PING)
  rCommand(CalibServoDrive,1)
  rCommand(EnableCounters,1)
  rCommand(SetSpeed, 95) // speed
  rCommand(SetSlowDownSpeed,85)
  rCommand(SetSlowDown2,65)
  rCommand(SetClicksPerDiam,73)
  rCommand(SetClicksPer90,42) 
  rCommand(SetBumpDist,10)
  rCommand(SetProxDist,20)
  rCommand(SetRotationTime,10) 
  rCommand(SetMoveTime,7) 
  rCommand(SetRightStopOffset,128)
  rCommand(SetLeftStopOffset,128)
  rCommand(SetReducForwRight,0)
  rCommand(SetReducForwLeft,0)
  rCommand(SetReducBackRight,0)
  rCommand(SetReducBackLeft,0)
  rCommand(SetMotorRamp,3)
  rCommand(SetCCdivisor,8)
  rCommand(SetDriveServoDir,1)
  rCommand(SetDriveServoWidth,80)//60
  rCommand(SetTurnToMax,70)
  rCommand(SetTurnToMin,40)
  rCommand(SetTurnStyle,50)
  rCommand(SetRobotAngle, 0)
  rCommand(SetRROStimeout, 15)
  rCommand(SetBeaconTime,35)
  rCommand(SetBeaconSpeed,50)
    
  // fix turn to half angle 180 difference (speed too)
return






