/*
NOTE: This program is intended to run in the BACKGROUND under control of the MAIN
      program.  See the book, Arlo: The Robot You've Always Wanted, for details.
      
Data in array is as follows
 0 - Left Shoulder L/R
 1 - Left Shoulder U/D
 2 - Left Elbow
 3 - Left Wrist
 4 - Left Wrist L/R
 5 - Left Hand

 6 - Right Shoulder L/R
 7 - Right Shoulder U/D
 8 - Right Elbow
 9 - Right Wrist
10 - Right Wrist L/R
11 - Right Hand

12 - Turret U/D
13 - Turret L/R

14 - Finger data
15 - US Range Data (1" units)
16 - IR Range Data (1" units)
17 - Limit (for scan)
18 - Request Scan  Bit 0-IR  Bit 1-ultrasonic   Bit2: 1-Vert, 0-Horz   
19 - Request Close Hand (1-left, 2-right)
20 - Request Open Hand (1-left, 2-right)
21 - Request Move Arms to Home Position  (1)
22 - Terminate Program
23 - Request Horizontal Laser Scan
24 - Reserved for expansion
25-38 Detailed Scan Data



 placing a number in a servo position will cause that motor to move to
 that position
 
 IR, US, and Finger Data will ALWAYS BE UPDATED

 placing -1 in CloseLeft, or CloseRight will cause it to close till switch activates
    both In/Out arrays are updated accordingly as well as the Close variables
 placing -1 in any of the SCAN variables will cause that scan to be made
 (7 positions are scanned at Close, Medium, Long, or Horizontal Range.
 If EITHER sensor finds an object LESS THAN the LIMIT VALUE then that bit 
 will be ON in the returned data position - LSB is Robot's Right Side)
 
 When Master Saves array it should be ReNamed ProcessNow
 When Slave performs actions and updates array it should be ReNamed DataReady
 
 When Slave starts, it should ensure that the array is named DataReady
 */

Main:  // this is the SLAVE program, SpeechControlDemo.bas is the MASTER program
  gosub Init 
  while TRUE
    repeat
    until FileExists ("ProcessNow")
    gosub Process
    FileRename("ProcessNow","DataReady")
  wend
end
 
Init:
  SetCommPort 7
  SerOut Char(0xAA)
  // The follow array data is the same as the original array data in the main program
  Data P;    1270,1332,1350,1257,1350,1674,1400,1330,1400,1375,1375,1220,1630,1780
  Data P;    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  
  Dim Last[14],Orig[14]
  for i=0 to 13
    Last[i]=P[i]
    Orig[i]=P[i]
  next
  // Horizontal turret positions for R to L scan
  Data H;    800,1075,1350,1624,1898,2173,2448
  Data V;    896,1073,1250,1427,1604,1780,1950 
  // vertical turret postions above
  ClosedLeft = 2200   // fully closed
  ClosedRight = 1725
  HalfClosedLeft = 1739
  HalfClosedRight = 1292
  OpenLeft = 1674
  OpenRight = 1220
  if FileExists("ProcessNow") then FileDelete("ProcessNow")
  mWrite P,"DataReady"
  LHLF = 4
  LHRF = 1
  LHacross = 8
  LHswitch = 16
  RHLF = 64
  RHRF = 2
  RHacross = 128
  RHswitch = 32
  gosub HomePosition
return

HomePosition:
  // return both arms (and turret) to their original home positions
  // first rotate left arm to prevent collisions
  call MoveToHome(0,0)
  // then position rest of left arm
  call MoveToHome(1,5)
  // now do right arm
  call MoveToHome(6,6)
  call MoveToHome(7,13)
  P[21]=0
return
 
Process:
  delay 1
  mRead P,"ProcessNow"
  if P[22] = 1 then mWrite P,"ProcessNow" \ end 
  if P[21] = 1 // Request for Home Position 
    gosub HomePosition
    mWrite P,"ProcessNow"
    return
  endif  
  gosub MoveServos
  if P[18] then gosub Scan
  if P[22] = 1 then gosub LaserScan
  if P[19] = 1 then gosub CloseLeftHand
  if P[19] = 2 then gosub CloseRightHand   
  if P[20] = 1 then gosub OpenLeftHand
  if P[20] = 2 then gosub OpenRightHand 
  mWrite P,"ProcessNow"
return

CloseLeftHand:
  // close half way
  ih = HalfClosedLeft // half closed
  SerOut char(0x84),char(5+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f)
  gosub ReadSensors
  // close till switch contact
  for ih = HalfClosedLeft to ClosedLeft step 15
    SerOut char(0x84),char(5+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f) 
    gosub ReadSensors
    if P[14]&LHswitch then break
  next
  ih-=10  //back up just a little on grip pressure
  if ih>1900 then ih -=10  // open more for small objects
  SerOut char(0x84),char(5+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f) 
  gosub ReadSensors
  Last[5]=ih
  P[5] = ih  
  P[19]=0
return

CloseRightHand:
  // close half way
  ih = HalfClosedRight
  SerOut char(0x84),char(11+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f)
  gosub ReadSensors
  // close till switch contact
  for ih = HalfClosedRight to ClosedRight step 15
    SerOut char(0x84),char(11+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f) 
    gosub ReadSensors
    if P[14]&RHswitch then break
  next
  ih-=10
  if ih>1550 then ih-=10
  SerOut char(0x84),char(11+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f) 
  gosub ReadSensors
  Last[11]=ih
  P[11] = ih 
  P[19]=0
return

OpenLeftHand:
  ih = OpenLeft
  SerOut char(0x84),char(5+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f)
  gosub ReadSensors
  Last[5]=ih
  P[5] = ih  
  P[20]=0
return

OpenRightHand:
  ih = OpenRight
  SerOut char(0x84),char(11+2),char(4*ih&0x7f),char((4*ih>>7)&0x7f)
  gosub ReadSensors
  Last[11]=ih
  P[11] = ih 
  P[20]=0  
return

Scan:
  LimVal = P[17]
  IR = P[18]&1
  USonic = P[18]&2 
  bit=1
  Answer = 0
  for ii=0 to 6
    if P[18]&4  // Vertical scan
      SerOut char(0x84),char(13+2),char(4*V[ii]&0x7f),char( (4*V[ii]>>7)&0x7f)
    else  // horizontal scan     
      SerOut char(0x84),char(12+2),char(4*H[ii]&0x7f),char( (4*H[ii]>>7)&0x7f)
    endif 
    if ii=0 then gosub ReadSensors\delay 200 // stabilize first position      
    // Read sensors and convert to inches and compare to LimVal
    gosub ReadSensors
    if (IR and (P[16]<LimVal))  or (USonic and (P[15]<LimVal))
      Answer+=bit     
    endif 
    bit = bit<<1 
    // store full data in Array, Ultrasonic first, then IR
    P[25+ii]=P[15]
    P[32+ii]=P[16]   
  next
  P[17] = Answer
  if P[18]&4
    SerOut char(0x84),char(13+2),char(4*V[3]&0x7f),char( (4*V[3]>>7)&0x7f)
  else  // horizontal scan     
    SerOut char(0x84),char(12+2),char(4*H[3]&0x7f),char( (4*H[3]>>7)&0x7f)
  endif
  ServPos = Orig[12]
  SerOut char(0x84),char(12+2),char(4*ServPos&0x7f),char( (4*ServPos>>7)&0x7f)
  P[18]=0   
return

LaserScan:
  // 13 positions (every 15 deg around front half of robot)
  StartPos = H[0]
  SerOut char(0x84),char(12+2),char(4*StartPos&0x7f),char( (4*StartPos>>7)&0x7f)
  gosub ReadSensors 
  delay 200
  IncPos = (H[6]-H[0])/13.0
  for ii = 0 to 12
    ServPos = round(StartPos+(ii*IncPos))
    if ServPos>H[6] then ServPos=H[6]
    SerOut char(0x84),char(12+2),char(4*ServPos&0x7f),char( (4*ServPos>>7)&0x7f)
    gosub ReadSensors
    P[25+ii]=P[16]
  next
  ServPos = Orig[12]
  SerOut char(0x84),char(12+2),char(4*ServPos&0x7f),char( (4*ServPos>>7)&0x7f)
  P[23]=0 
return

MoveServos:
  for i = 0 to 13
    if P[i]!=Last[i]
      SerOut char(0x84),char(i+2),char(4*P[i]&0x7f),char( (4*P[i]>>7)&0x7f)
      Last[i] = P[i]
    endif 
  next
  gosub ReadSensors
return

sub MoveToHome(F,L) // first to last
  for i = F to L
    P[i]=Orig[i]
    SerOut char(0x84),char(i+2),char(4*P[i]&0x7f),char( (4*P[i]>>7)&0x7f)
    Last[i] = P[i] 
  next
  gosub ReadSensors
return

ReadSensors:
  Laser = TRUE // Set to false for Pololu Sharp IR ranger 
               // Set to true for Parallax SF02 Laser ranger
  FingerTotal=255
  IRtotal = 0
  UStotal = 0 
  for k=1 to 2
    // watch for sync bytes
    repeat
      SerOut char(0xA7),char(0) // start script 0
      repeat
        SerBytesIn 1,DataRead,NumberRead
      until ascii(DataRead) = 1
      SerBytesIn 1,DataRead,NumberRead
    until Ascii(DataRead) = 2     
    // then read the actual data
    SerBytesIn 3,DataRead,NumberRead
    for j=1 to 3
      // place in positions 14,15, and 16
      P[13+j] = ascii(substring(DataRead,j,1))
    next 
    UStotal+=P[15]   
    IRtotal+=P[16]   
    FingerTotal=FingerTotal&P[14] // reduce false readings 
  next
  P[15]=UStotal/2  // /4
  P[16]=IRtotal/2
  // convert ultrasonic to inches
  P[15] = round(.48*P[15]) //round(P[15]*.48)  
  if P[15]>60 then P[15]=255  
  if not Laser   // Laser data does not require ANY changes
    // IR is very nonlinear
    // approximate using several linear equations
    if P[16]>=86
       P[16]=round(33-.1765*P[16])
    elseif P[16]>63
      P[16]=round(44-.3*P[16])
    elseif P[16]>=45
      P[16]=round(63.4-.61*P[16])
    elseif P[16]>=28
      P[16]=round(101.5-1.47*P[16])
    else
      P[16]=255
    endif 
  endif
  P[14]=FingerTotal
  // shows data for clarity - may be removed     
  for j= 1 to 3
    xyString 100,100+j*20, "<",P[13+j],"    >"
  next
return
